@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998-2005, 2008, 2009-2016
+ * Copyright (c) 1996, 1998-2005, 2008, 2009-2017
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -202,6 +202,11 @@
 # endif
 #endif
 
+/* For pipe2() emulation. */
+#if !defined(HAVE_PIPE2) && defined(O_NONBLOCK) && !defined(O_CLOEXEC)
+# define O_CLOEXEC	0x80000000
+#endif
+
 /*
  * BSD defines these in <sys/param.h> but we don't include that anymore.
  */
@@ -220,6 +225,13 @@
 #undef ISSET
 #define ISSET(t, f)     ((t) & (f))
 
+/*
+ * Some systems define this in <sys/param.h> but we don't include that anymore.
+ */
+#ifndef howmany
+# define howmany(x, y)	(((x) + ((y) - 1)) / (y))
+#endif
+
 /*
  * Simple isblank() macro and function for systems without it.
  */
@@ -236,20 +248,6 @@ __dso_public int isblank(int);
 # define HAVE_INNETGR 1
 #endif /* HAVE__INNETGR */
 
-/*
- * Add IRIX-like sigaction_t for those without it.
- * SA_RESTART is not required by POSIX; SunOS has SA_INTERRUPT instead.
- */
-#ifndef HAVE_SIGACTION_T
-typedef struct sigaction sigaction_t;
-#endif
-#ifndef SA_INTERRUPT
-# define SA_INTERRUPT	0
-#endif
-#ifndef SA_RESTART
-# define SA_RESTART	0
-#endif
-
 /*
  * The nitems macro may be defined in sys/param.h
  */
@@ -266,7 +264,7 @@ typedef struct sigaction sigaction_t;
 #endif
 
 #if !defined(HAVE_KILLPG) && !defined(killpg)
-# define killpg(s)	kill(-(s))
+# define killpg(p, s)	kill(-(p), (s))
 #endif
 
 /*
@@ -468,6 +466,11 @@ __dso_public int sudo_mkstemps(char *path, int slen);
 # undef mkstemps
 # define mkstemps(_a, _b) sudo_mkstemps((_a), (_b))
 #endif /* !HAVE_MKDTEMP || !HAVE_MKSTEMPS */
+#ifndef HAVE_NANOSLEEP
+__dso_public int sudo_nanosleep(const struct timespec *timeout, struct timespec *remainder);
+#undef nanosleep
+# define nanosleep(_a, _b) sudo_nanosleep((_a), (_b))
+#endif
 #ifndef HAVE_PW_DUP
 __dso_public struct passwd *sudo_pw_dup(const struct passwd *pw);
 # undef pw_dup
@@ -508,5 +511,10 @@ __dso_public void sudo_vsyslog(int pri, const char *fmt, va_list ap);
 # undef vsyslog
 # define vsyslog(_a, _b, _c) sudo_vsyslog((_a), (_b), (_c))
 #endif /* HAVE_VSYSLOG */
+#ifndef HAVE_PIPE2
+__dso_public int sudo_pipe2(int fildes[2], int flags);
+# undef pipe2
+# define pipe2(_a, _b) sudo_pipe2((_a), (_b))
+#endif /* HAVE_PIPE2 */
 
 #endif /* SUDO_COMPAT_H */
