@@ -22,38 +22,17 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #include <fcntl.h>
 #include <limits.h>
 #ifdef HAVE_PSTAT_GETPROC
 # include <sys/pstat.h>
 #else
-# ifdef HAVE_DIRENT_H
-#  include <dirent.h>
-#  define NAMLEN(dirent) strlen((dirent)->d_name)
-# else
-#  define dirent direct
-#  define NAMLEN(dirent) (dirent)->d_namlen
-#  ifdef HAVE_SYS_NDIR_H
-#   include <sys/ndir.h>
-#  endif
-#  ifdef HAVE_SYS_DIR_H
-#   include <sys/dir.h>
-#  endif
-#  ifdef HAVE_NDIR_H
-#   include <ndir.h>
-#  endif
-# endif
+# include <dirent.h>
 #endif
 
 #include "sudo_compat.h"
+#include "pathnames.h"
 
 #ifndef _POSIX_OPEN_MAX
 # define _POSIX_OPEN_MAX	20
@@ -125,7 +104,7 @@ sudo_closefrom(int lowfd)
 
     /* Use /proc/self/fd (or /dev/fd on FreeBSD) if it exists. */
 # if defined(__FreeBSD__) || defined(__APPLE__)
-    path = "/dev/fd";
+    path = _PATH_DEV "fd";
 # else
     path = "/proc/self/fd";
 # endif
