@@ -1,6 +1,5 @@
 /*
- * Copyright (c) 2007, 2013-2015
- *	Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2007, 2013-2016 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -46,15 +45,16 @@
  *       could also make comment char and line continuation configurable
  */
 ssize_t
-sudo_parseln_v1(char **bufp, size_t *bufsizep, unsigned int *lineno, FILE *fp)
+sudo_parseln_v2(char **bufp, size_t *bufsizep, unsigned int *lineno, FILE *fp, int flags)
 {
     size_t linesize = 0, total = 0;
     ssize_t len;
     char *cp, *line = NULL;
-    bool continued;
+    bool continued, comment;
     debug_decl(sudo_parseln, SUDO_DEBUG_UTIL)
 
     do {
+	comment = false;
 	continued = false;
 	len = getline(&line, &linesize, fp);
 	if (len == -1)
@@ -68,11 +68,17 @@ sudo_parseln_v1(char **bufp, size_t *bufsizep, unsigned int *lineno, FILE *fp)
 
 	/* Remove comments or check for line continuation (but not both) */
 	if ((cp = strchr(line, '#')) != NULL) {
-	    *cp = '\0';
-	    len = (ssize_t)(cp - line);
-	} else if (len > 0 && line[len - 1] == '\\' && (len == 1 || line[len - 2] != '\\')) {
-	    line[--len] = '\0';
-	    continued = true;
+	    if (cp == line || !ISSET(flags, PARSELN_COMM_BOL)) {
+		*cp = '\0';
+		len = (ssize_t)(cp - line);
+		comment = true;
+	    }
+	}
+	if (!comment && !ISSET(flags, PARSELN_CONT_IGN)) {
+	    if (len > 0 && line[len - 1] == '\\' && (len == 1 || line[len - 2] != '\\')) {
+		line[--len] = '\0';
+		continued = true;
+	    }
 	}
 
 	/* Trim leading and trailing whitespace */
@@ -117,3 +123,9 @@ sudo_parseln_v1(char **bufp, size_t *bufsizep, unsigned int *lineno, FILE *fp)
 	debug_return_ssize_t(-1);
     debug_return_ssize_t(total);
 }
+
+ssize_t
+sudo_parseln_v1(char **bufp, size_t *bufsizep, unsigned int *lineno, FILE *fp)
+{
+    return sudo_parseln_v2(bufp, bufsizep, lineno, fp, 0);
+}
