@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2016 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2011-2017 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -243,8 +243,9 @@ bad:
  * If subsystem names are specified they override the default values.
  * NOTE: subsystems must not be freed by caller unless deregistered.
  * Sets the active instance to the newly registered instance.
- * Returns instance index on success or SUDO_DEBUG_INSTANCE_INITIALIZER
- * on failure.
+ * Returns instance index on success, SUDO_DEBUG_INSTANCE_INITIALIZER
+ * if no debug files are specified and SUDO_DEBUG_INSTANCE_ERROR
+ * on error.
  */
 int
 sudo_debug_register_v1(const char *program, const char *const subsystems[],
@@ -264,7 +265,7 @@ sudo_debug_register_v1(const char *program, const char *const subsystems[],
 	subsystems = sudo_debug_default_subsystems;
     } else if (ids == NULL) {
 	/* If subsystems are specified we must have ids[] too. */
-	return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	return SUDO_DEBUG_INSTANCE_ERROR;
     }
 
     /* Search for existing instance. */
@@ -302,17 +303,17 @@ sudo_debug_register_v1(const char *program, const char *const subsystems[],
 	if (idx == SUDO_DEBUG_INSTANCE_MAX) {
 	    /* XXX - realloc? */
 	    sudo_warnx_nodebug("too many debug instances (max %d)", SUDO_DEBUG_INSTANCE_MAX);
-	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	    return SUDO_DEBUG_INSTANCE_ERROR;
 	}
 	if (idx != sudo_debug_last_instance + 1 && idx != free_idx) {
 	    sudo_warnx_nodebug("%s: instance number mismatch: expected %d or %d, got %d", __func__, sudo_debug_last_instance + 1, free_idx, idx);
-	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	    return SUDO_DEBUG_INSTANCE_ERROR;
 	}
 	if ((instance = malloc(sizeof(*instance))) == NULL)
-	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	    return SUDO_DEBUG_INSTANCE_ERROR;
 	if ((instance->program = strdup(program)) == NULL) {
 	    free(instance);
-	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	    return SUDO_DEBUG_INSTANCE_ERROR;
 	}
 	instance->subsystems = subsystems;
 	instance->subsystem_ids = ids;
@@ -451,6 +452,19 @@ sudo_debug_exit_long_v1(const char *func, const char *file, int line,
 	"<- %s @ %s:%d := %ld", func, file, line, ret);
 }
 
+void
+sudo_debug_exit_id_t_v1(const char *func, const char *file, int line,
+    int subsys, id_t ret)
+{
+#if SIZEOF_ID_T == 8
+    sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
+	"<- %s @ %s:%d := %lld", func, file, line, (long long)ret);
+#else
+    sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
+	"<- %s @ %s:%d := %d", func, file, line, (int)ret);
+#endif
+}
+
 void
 sudo_debug_exit_size_t_v1(const char *func, const char *file, int line,
     int subsys, size_t ret)
@@ -467,6 +481,19 @@ sudo_debug_exit_ssize_t_v1(const char *func, const char *file, int line,
 	"<- %s @ %s:%d := %zd", func, file, line, ret);
 }
 
+void
+sudo_debug_exit_time_t_v1(const char *func, const char *file, int line,
+    int subsys, time_t ret)
+{
+#if SIZEOF_TIME_T == 8
+    sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
+	"<- %s @ %s:%d := %lld", func, file, line, (long long)ret);
+#else
+    sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
+	"<- %s @ %s:%d := %d", func, file, line, (int)ret);
+#endif
+}
+
 void
 sudo_debug_exit_bool_v1(const char *func, const char *file, int line,
     int subsys, bool ret)
