@@ -21,33 +21,15 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#include <termios.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <termios.h>		/* for struct winsize on HP-UX */
 #include <limits.h>
 
 #include "sudo_compat.h"
 #include "sudo_debug.h"
 #include "sudo_util.h"
 
-/* Compatibility with older tty systems. */
-#if !defined(TIOCGWINSZ) && defined(TIOCGSIZE)
-# define TIOCGWINSZ	TIOCGSIZE
-# define winsize	ttysize
-# define ws_col		ts_cols
-# define ws_row		ts_lines
-#endif
-
-#ifdef TIOCGWINSZ
 static int
 get_ttysize_ioctl(int *rowp, int *colp)
 {
@@ -62,13 +44,6 @@ get_ttysize_ioctl(int *rowp, int *colp)
     }
     debug_return_int(-1);
 }
-#else
-static int
-get_ttysize_ioctl(int *rowp, int *colp)
-{
-    return -1;
-}
-#endif /* TIOCGWINSZ */
 
 void
 sudo_get_ttysize_v1(int *rowp, int *colp)
