@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1996,1998-2005, 2007-2015
+ * Copyright (c) 1993-1996,1998-2005, 2007-2016
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -46,26 +46,76 @@
 static bool display_lecture(int);
 static struct passwd *get_authpw(int);
 
+struct getpass_closure {
+    void *cookie;
+    struct passwd *auth_pw;
+};
+
+/*
+ * Called when getpass is suspended so we can drop the lock.
+ */
+static int
+getpass_suspend(int signo, void *vclosure)
+{
+    struct getpass_closure *closure = vclosure;
+
+    timestamp_close(closure->cookie);
+    closure->cookie = NULL;
+    return 0;
+}
+
+/*
+ * Called when getpass is resumed so we can reacquire the lock.
+ */
+static int
+getpass_resume(int signo, void *vclosure)
+{
+    struct getpass_closure *closure = vclosure;
+
+    closure->cookie = timestamp_open(user_name, user_sid);
+    if (closure->cookie == NULL)
+	return -1;
+    if (!timestamp_lock(closure->cookie, closure->auth_pw))
+	return -1;
+    return 0;
+}
+
 /*
  * Returns true if the user successfully authenticates, false if not
- * or -1 on error.
+ * or -1 on fatal error.
  */
 static int
 check_user_interactive(int validated, int mode, struct passwd *auth_pw)
 {
-    int status, rval = -1;
+    struct sudo_conv_callback cb, *callback = NULL;
+    struct getpass_closure closure;
+    int status = TS_ERROR;
+    int ret = -1;
     char *prompt;
     bool lectured;
     debug_decl(check_user_interactive, SUDOERS_DEBUG_AUTH)
 
-    /* Always need a password when -k was specified with the command. */
-    if (ISSET(mode, MODE_IGNORE_TICKET))
-	SET(validated, FLAG_CHECK_USER);
+    /* Setup closure for getpass_{suspend,resume} */
+    closure.auth_pw = auth_pw;
+    closure.cookie = NULL;
+    sudo_pw_addref(closure.auth_pw);
 
-    if (build_timestamp(auth_pw) == -1)
-	goto done;
+    /* Open, lock and read time stamp file if we are using it. */
+    if (!ISSET(mode, MODE_IGNORE_TICKET)) {
+	/* Open time stamp file and check its status. */
+	closure.cookie = timestamp_open(user_name, user_sid);
+	if (timestamp_lock(closure.cookie, closure.auth_pw))
+	    status = timestamp_status(closure.cookie, closure.auth_pw);
+
+	/* Construct callback for getpass function. */
+	memset(&cb, 0, sizeof(cb));
+	cb.version = SUDO_CONV_CALLBACK_VERSION;
+	cb.closure = &closure;
+	cb.on_suspend = getpass_suspend;
+	cb.on_resume = getpass_resume;
+	callback = &cb;
+    }
 
-    status = timestamp_status(auth_pw);
     switch (status) {
     case TS_FATAL:
 	/* Fatal error (usually setuid failure), unsafe to proceed. */
@@ -74,9 +124,11 @@ check_user_interactive(int validated, int mode, struct passwd *auth_pw)
     case TS_CURRENT:
 	/* Time stamp file is valid and current. */
 	if (!ISSET(validated, FLAG_CHECK_USER)) {
-	    rval = true;
+	    ret = true;
 	    break;
 	}
+	sudo_debug_printf(SUDO_DEBUG_INFO,
+	    "%s: check user flag overrides time stamp", __func__);
 	/* FALLTHROUGH */
 
     default:
@@ -92,27 +144,29 @@ check_user_interactive(int validated, int mode, struct passwd *auth_pw)
 
 	/* Expand any escapes in the prompt. */
 	prompt = expand_prompt(user_prompt ? user_prompt : def_passprompt,
-	    auth_pw->pw_name);
+	    closure.auth_pw->pw_name);
 	if (prompt == NULL)
 	    goto done;
 
-	rval = verify_user(auth_pw, prompt, validated);
-	if (rval == true && lectured) {
-	    if (set_lectured() == -1)
-		rval = -1;
-	}
+	ret = verify_user(closure.auth_pw, prompt, validated, callback);
+	if (ret == true && lectured)
+	    (void)set_lectured();	/* lecture error not fatal */
 	free(prompt);
 	break;
     }
 
-    /* Only update timestamp if user was validated. */
-    if (rval == true && ISSET(validated, VALIDATE_SUCCESS) &&
-	!ISSET(mode, MODE_IGNORE_TICKET) && status != TS_ERROR) {
-	if (update_timestamp(auth_pw) == -1)
-	    rval = -1;
-    }
+    /*
+     * Only update time stamp if user was validated.
+     * Failure to update the time stamp is not a fatal error.
+     */
+    if (ret == true && ISSET(validated, VALIDATE_SUCCESS) && status != TS_ERROR)
+	(void)timestamp_update(closure.cookie, closure.auth_pw);
 done:
-    debug_return_int(rval);
+    if (closure.cookie != NULL)
+	timestamp_close(closure.cookie);
+    sudo_pw_delref(closure.auth_pw);
+
+    debug_return_int(ret);
 }
 
 /*
@@ -123,7 +177,7 @@ int
 check_user(int validated, int mode)
 {
     struct passwd *auth_pw;
-    int rval = -1;
+    int ret = -1;
     debug_decl(check_user, SUDOERS_DEBUG_AUTH)
 
     /*
@@ -140,7 +194,10 @@ check_user(int validated, int mode)
      * If the user is not changing uid/gid, no need for a password.
      */
     if (!def_authenticate || user_is_exempt()) {
-	rval = true;
+	sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %s", __func__,
+	    !def_authenticate ? "authentication disabled" :
+	    "user exempt from authentication");
+	ret = true;
 	goto done;
     }
     if (user_uid == 0 || (user_uid == runas_pw->pw_uid &&
@@ -152,18 +209,20 @@ check_user(int validated, int mode)
 	if (runas_privs == NULL && runas_limitprivs == NULL)
 #endif
 	{
-	    rval = true;
+	    sudo_debug_printf(SUDO_DEBUG_INFO,
+		"%s: user running command as self", __func__);
+	    ret = true;
 	    goto done;
 	}
     }
 
-    rval = check_user_interactive(validated, mode, auth_pw);
+    ret = check_user_interactive(validated, mode, auth_pw);
 
 done:
     sudo_auth_cleanup(auth_pw);
     sudo_pw_delref(auth_pw);
 
-    debug_return_int(rval);
+    debug_return_int(ret);
 }
 
 /*
@@ -192,7 +251,7 @@ display_lecture(int status)
 	    buf[nread] = '\0';
 	    msg.msg_type = SUDO_CONV_ERROR_MSG;
 	    msg.msg = buf;
-	    sudo_conv(1, &msg, &repl);
+	    sudo_conv(1, &msg, &repl, NULL);
 	}
 	fclose(fp);
     } else {
@@ -203,7 +262,7 @@ display_lecture(int status)
 	    "    #1) Respect the privacy of others.\n"
 	    "    #2) Think before you type.\n"
 	    "    #3) With great power comes great responsibility.\n\n");
-	sudo_conv(1, &msg, &repl);
+	sudo_conv(1, &msg, &repl, NULL);
     }
     debug_return_bool(true);
 }
@@ -214,12 +273,12 @@ display_lecture(int status)
 bool
 user_is_exempt(void)
 {
-    bool rval = false;
+    bool ret = false;
     debug_decl(user_is_exempt, SUDOERS_DEBUG_AUTH)
 
     if (def_exempt_group)
-	rval = user_in_group(sudo_user.pw, def_exempt_group);
-    debug_return_bool(rval);
+	ret = user_in_group(sudo_user.pw, def_exempt_group);
+    debug_return_bool(ret);
 }
 
 /*
