@@ -134,81 +134,101 @@
 #define def_type                (sudo_defs_table[I_TYPE].sd_un.str)
 #define I_ENV_FILE              67
 #define def_env_file            (sudo_defs_table[I_ENV_FILE].sd_un.str)
-#define I_SUDOERS_LOCALE        68
+#define I_RESTRICTED_ENV_FILE   68
+#define def_restricted_env_file (sudo_defs_table[I_RESTRICTED_ENV_FILE].sd_un.str)
+#define I_SUDOERS_LOCALE        69
 #define def_sudoers_locale      (sudo_defs_table[I_SUDOERS_LOCALE].sd_un.str)
-#define I_VISIBLEPW             69
+#define I_VISIBLEPW             70
 #define def_visiblepw           (sudo_defs_table[I_VISIBLEPW].sd_un.flag)
-#define I_PWFEEDBACK            70
+#define I_PWFEEDBACK            71
 #define def_pwfeedback          (sudo_defs_table[I_PWFEEDBACK].sd_un.flag)
-#define I_FAST_GLOB             71
+#define I_FAST_GLOB             72
 #define def_fast_glob           (sudo_defs_table[I_FAST_GLOB].sd_un.flag)
-#define I_UMASK_OVERRIDE        72
+#define I_UMASK_OVERRIDE        73
 #define def_umask_override      (sudo_defs_table[I_UMASK_OVERRIDE].sd_un.flag)
-#define I_LOG_INPUT             73
+#define I_LOG_INPUT             74
 #define def_log_input           (sudo_defs_table[I_LOG_INPUT].sd_un.flag)
-#define I_LOG_OUTPUT            74
+#define I_LOG_OUTPUT            75
 #define def_log_output          (sudo_defs_table[I_LOG_OUTPUT].sd_un.flag)
-#define I_COMPRESS_IO           75
+#define I_COMPRESS_IO           76
 #define def_compress_io         (sudo_defs_table[I_COMPRESS_IO].sd_un.flag)
-#define I_USE_PTY               76
+#define I_USE_PTY               77
 #define def_use_pty             (sudo_defs_table[I_USE_PTY].sd_un.flag)
-#define I_GROUP_PLUGIN          77
+#define I_GROUP_PLUGIN          78
 #define def_group_plugin        (sudo_defs_table[I_GROUP_PLUGIN].sd_un.str)
-#define I_IOLOG_DIR             78
+#define I_IOLOG_DIR             79
 #define def_iolog_dir           (sudo_defs_table[I_IOLOG_DIR].sd_un.str)
-#define I_IOLOG_FILE            79
+#define I_IOLOG_FILE            80
 #define def_iolog_file          (sudo_defs_table[I_IOLOG_FILE].sd_un.str)
-#define I_SET_UTMP              80
+#define I_SET_UTMP              81
 #define def_set_utmp            (sudo_defs_table[I_SET_UTMP].sd_un.flag)
-#define I_UTMP_RUNAS            81
+#define I_UTMP_RUNAS            82
 #define def_utmp_runas          (sudo_defs_table[I_UTMP_RUNAS].sd_un.flag)
-#define I_PRIVS                 82
+#define I_PRIVS                 83
 #define def_privs               (sudo_defs_table[I_PRIVS].sd_un.str)
-#define I_LIMITPRIVS            83
+#define I_LIMITPRIVS            84
 #define def_limitprivs          (sudo_defs_table[I_LIMITPRIVS].sd_un.str)
-#define I_EXEC_BACKGROUND       84
+#define I_EXEC_BACKGROUND       85
 #define def_exec_background     (sudo_defs_table[I_EXEC_BACKGROUND].sd_un.flag)
-#define I_PAM_SERVICE           85
+#define I_PAM_SERVICE           86
 #define def_pam_service         (sudo_defs_table[I_PAM_SERVICE].sd_un.str)
-#define I_PAM_LOGIN_SERVICE     86
+#define I_PAM_LOGIN_SERVICE     87
 #define def_pam_login_service   (sudo_defs_table[I_PAM_LOGIN_SERVICE].sd_un.str)
-#define I_PAM_SETCRED           87
+#define I_PAM_SETCRED           88
 #define def_pam_setcred         (sudo_defs_table[I_PAM_SETCRED].sd_un.flag)
-#define I_PAM_SESSION           88
+#define I_PAM_SESSION           89
 #define def_pam_session         (sudo_defs_table[I_PAM_SESSION].sd_un.flag)
-#define I_MAXSEQ                89
+#define I_MAXSEQ                90
 #define def_maxseq              (sudo_defs_table[I_MAXSEQ].sd_un.uival)
-#define I_USE_NETGROUPS         90
+#define I_USE_NETGROUPS         91
 #define def_use_netgroups       (sudo_defs_table[I_USE_NETGROUPS].sd_un.flag)
-#define I_SUDOEDIT_CHECKDIR     91
+#define I_SUDOEDIT_CHECKDIR     92
 #define def_sudoedit_checkdir   (sudo_defs_table[I_SUDOEDIT_CHECKDIR].sd_un.flag)
-#define I_SUDOEDIT_FOLLOW       92
+#define I_SUDOEDIT_FOLLOW       93
 #define def_sudoedit_follow     (sudo_defs_table[I_SUDOEDIT_FOLLOW].sd_un.flag)
-#define I_ALWAYS_QUERY_GROUP_PLUGIN 93
+#define I_ALWAYS_QUERY_GROUP_PLUGIN 94
 #define def_always_query_group_plugin (sudo_defs_table[I_ALWAYS_QUERY_GROUP_PLUGIN].sd_un.flag)
-#define I_NETGROUP_TUPLE        94
+#define I_NETGROUP_TUPLE        95
 #define def_netgroup_tuple      (sudo_defs_table[I_NETGROUP_TUPLE].sd_un.flag)
-#define I_IGNORE_AUDIT_ERRORS   95
+#define I_IGNORE_AUDIT_ERRORS   96
 #define def_ignore_audit_errors (sudo_defs_table[I_IGNORE_AUDIT_ERRORS].sd_un.flag)
-#define I_IGNORE_IOLOG_ERRORS   96
+#define I_IGNORE_IOLOG_ERRORS   97
 #define def_ignore_iolog_errors (sudo_defs_table[I_IGNORE_IOLOG_ERRORS].sd_un.flag)
-#define I_IGNORE_LOGFILE_ERRORS 97
+#define I_IGNORE_LOGFILE_ERRORS 98
 #define def_ignore_logfile_errors (sudo_defs_table[I_IGNORE_LOGFILE_ERRORS].sd_un.flag)
-#define I_MATCH_GROUP_BY_GID    98
+#define I_MATCH_GROUP_BY_GID    99
 #define def_match_group_by_gid  (sudo_defs_table[I_MATCH_GROUP_BY_GID].sd_un.flag)
-#define I_SYSLOG_MAXLEN         99
+#define I_SYSLOG_MAXLEN         100
 #define def_syslog_maxlen       (sudo_defs_table[I_SYSLOG_MAXLEN].sd_un.uival)
-#define I_IOLOG_USER            100
+#define I_IOLOG_USER            101
 #define def_iolog_user          (sudo_defs_table[I_IOLOG_USER].sd_un.str)
-#define I_IOLOG_GROUP           101
+#define I_IOLOG_GROUP           102
 #define def_iolog_group         (sudo_defs_table[I_IOLOG_GROUP].sd_un.str)
-#define I_IOLOG_MODE            102
+#define I_IOLOG_MODE            103
 #define def_iolog_mode          (sudo_defs_table[I_IOLOG_MODE].sd_un.mode)
+#define I_FDEXEC                104
+#define def_fdexec              (sudo_defs_table[I_FDEXEC].sd_un.tuple)
+#define I_IGNORE_UNKNOWN_DEFAULTS 105
+#define def_ignore_unknown_defaults (sudo_defs_table[I_IGNORE_UNKNOWN_DEFAULTS].sd_un.flag)
+#define I_COMMAND_TIMEOUT       106
+#define def_command_timeout     (sudo_defs_table[I_COMMAND_TIMEOUT].sd_un.ival)
+#define I_USER_COMMAND_TIMEOUTS 107
+#define def_user_command_timeouts (sudo_defs_table[I_USER_COMMAND_TIMEOUTS].sd_un.flag)
+#define I_IOLOG_FLUSH           108
+#define def_iolog_flush         (sudo_defs_table[I_IOLOG_FLUSH].sd_un.flag)
+#define I_SYSLOG_PID            109
+#define def_syslog_pid          (sudo_defs_table[I_SYSLOG_PID].sd_un.flag)
+#define I_TIMESTAMP_TYPE        110
+#define def_timestamp_type      (sudo_defs_table[I_TIMESTAMP_TYPE].sd_un.tuple)
 
 enum def_tuple {
 	never,
 	once,
 	always,
 	any,
-	all
+	all,
+	digest_only,
+	global,
+	ppid,
+	tty
 };
