@@ -164,8 +164,7 @@ static const char *initial_badenv_table[] = {
     "PYTHONUSERBASE",		/* python, per user site-packages directory */
     "RUBYLIB",			/* ruby, library load path */
     "RUBYOPT",			/* ruby, extra command line options */
-    "BASH_FUNC_*",		/* new-style bash functions */
-    "__BASH_FUNC<*",		/* new-style bash functions (Apple) */
+    "*=()*",			/* bash functions */
     NULL
 };
 
@@ -497,6 +496,7 @@ sudo_unsetenv_nodebug(const char *var)
 	    char **cur = ep;
 	    while ((*cur = *(cur + 1)) != NULL)
 		cur++;
+	    env.env_len--;
 	    /* Keep going, could be multiple instances of the var. */
 	} else {
 	    ep++;
@@ -569,30 +569,13 @@ static bool
 matches_env_list(const char *var, struct list_members *list, bool *full_match)
 {
     struct list_member *cur;
-    bool match = false;
     debug_decl(matches_env_list, SUDOERS_DEBUG_ENV)
 
     SLIST_FOREACH(cur, list, entries) {
-	size_t sep_pos, len = strlen(cur->value);
-	bool iswild = false;
-
-	/* Locate position of the '=' separator in var=value. */
-	sep_pos = strcspn(var, "=");
-
-	/* Deal with '*' wildcard at the end of the pattern. */
-	if (cur->value[len - 1] == '*') {
-	    len--;
-	    iswild = true;
-	}
-	if (strncmp(cur->value, var, len) == 0 &&
-	    (iswild || len == sep_pos || var[len] == '\0')) {
-	    /* If we matched past the '=', count as a full match. */
-	    *full_match = len > sep_pos + 1;
-	    match = true;
-	    break;
-	}
+	if (matches_env_pattern(cur->value, var, full_match))
+	    debug_return_bool(true);
     }
-    debug_return_bool(match);
+    debug_return_bool(false);
 }
 
 /*
@@ -708,24 +691,14 @@ matches_env_keep(const char *var, bool *full_match)
 static bool
 env_should_delete(const char *var)
 {
-    const char *cp;
     int delete_it;
     bool full_match = false;
     debug_decl(env_should_delete, SUDOERS_DEBUG_ENV);
 
-    /* Skip variables with values beginning with () (bash functions) */
-    if ((cp = strchr(var, '=')) != NULL) {
-	if (strncmp(cp, "=() ", 4) == 0) {
-	    delete_it = true;
-	    goto done;
-	}
-    }
-
     delete_it = matches_env_delete(var);
     if (!delete_it)
 	delete_it = matches_env_check(var, &full_match) == false;
 
-done:
     sudo_debug_printf(SUDO_DEBUG_INFO, "delete %s: %s",
 	var, delete_it ? "YES" : "NO");
     debug_return_bool(delete_it);
@@ -899,7 +872,7 @@ rebuild_env(void)
 #endif /* HAVE_LOGIN_CAP_H */
 #if defined(_AIX) || (defined(__linux__) && !defined(HAVE_PAM))
 	    /* Insert system-wide environment variables. */
-	    read_env_file(_PATH_ENVIRONMENT, true);
+	    read_env_file(_PATH_ENVIRONMENT, true, false);
 #endif
 	    for (ep = env.envp; *ep; ep++)
 		env_update_didvar(*ep, &didvar);
@@ -1170,7 +1143,7 @@ validate_env_vars(char * const env_vars[])
  * character are skipped.
  */
 bool
-read_env_file(const char *path, int overwrite)
+read_env_file(const char *path, bool overwrite, bool restricted)
 {
     FILE *fp;
     bool ret = true;
@@ -1205,6 +1178,15 @@ read_env_file(const char *path, int overwrite)
 	var_len = (size_t)(val - var);
 	val_len = strlen(++val);
 
+	/*
+	 * If the env file is restricted, apply env_check and env_keep
+	 * when env_reset is set or env_delete when it is not.
+	 */
+	if (restricted) {
+	    if (def_env_reset ? !env_should_keep(var) : env_should_delete(var))
+		continue;
+	}
+
 	/* Strip leading and trailing single/double quotes */
 	if ((val[0] == '\'' || val[0] == '\"') && val[0] == val[val_len - 1]) {
 	    val[val_len - 1] = '\0';
