@@ -18,18 +18,8 @@
 
 #include <sys/types.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
-# if defined(HAVE_MEMORY_H) && !defined(STDC_HEADERS)
-#  include <memory.h>
-# endif
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
@@ -172,7 +162,11 @@ expand_iolog_path(const char *prefix, const char *dir, const char *file,
     /* Expanded path must be <= PATH_MAX */
     if (prefix != NULL)
 	prelen = strlen(prefix);
-    dst = path = sudo_emalloc(prelen + PATH_MAX);
+    dst = path = malloc(prelen + PATH_MAX);
+    if (path == NULL) {
+	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	goto bad;
+    }
     *path = '\0';
     pathend = path + prelen + PATH_MAX;
 
@@ -282,6 +276,6 @@ expand_iolog_path(const char *prefix, const char *dir, const char *file,
 
     debug_return_str(path);
 bad:
-    sudo_efree(path);
+    free(path);
     debug_return_str(NULL);
 }
