@@ -14,9 +14,9 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifndef _SUDOERS_LINUX_AUDIT_H
-#define	_SUDOERS_LINUX_AUDIT_H
+#ifndef SUDOERS_LINUX_AUDIT_H
+#define	SUDOERS_LINUX_AUDIT_H
 
 int	linux_audit_command(char *argv[], int result);
 
-#endif /* _SUDOERS_LINUX_AUDIT_H */
+#endif /* SUDOERS_LINUX_AUDIT_H */
