@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005, 2007-2016 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2004-2005, 2007-2017 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -29,8 +29,9 @@
 #endif /* HAVE_STRINGS_H */
 #include <unistd.h>
 #include <ctype.h>
-#include <pwd.h>
 #include <grp.h>
+#include <pwd.h>
+#include <time.h>
 
 #include "sudoers.h"
 #include "parse.h"
@@ -145,12 +146,13 @@ sudo_file_setdefs(struct sudo_nss *nss)
 int
 sudo_file_lookup(struct sudo_nss *nss, int validated, int pwflag)
 {
-    int match, host_match, runas_match, cmnd_match;
+    int match, host_match, runas_match, cmnd_match, timeout;
     struct cmndspec *cs;
     struct cmndtag *tags = NULL;
     struct privilege *priv;
     struct userspec *us;
     struct member *matching_user;
+    time_t now;
     debug_decl(sudo_file_lookup, SUDOERS_DEBUG_NSS)
 
     if (nss->handle == NULL)
@@ -207,6 +209,7 @@ sudo_file_lookup(struct sudo_nss *nss, int validated, int pwflag)
     if (!set_perms(PERM_RUNAS))
 	debug_return_int(validated);
 
+    time(&now);
     match = UNSPEC;
     TAILQ_FOREACH_REVERSE(us, &userspecs, userspec_list, entries) {
 	if (userlist_matches(sudo_user.pw, &us->users) != ALLOW)
@@ -219,6 +222,14 @@ sudo_file_lookup(struct sudo_nss *nss, int validated, int pwflag)
 	    else
 		continue;
 	    TAILQ_FOREACH_REVERSE(cs, &priv->cmndlist, cmndspec_list, entries) {
+		if (cs->notbefore != UNSPEC) {
+		    if (now < cs->notbefore)
+			continue;
+		}
+		if (cs->notafter != UNSPEC) {
+		    if (now > cs->notafter)
+			continue;
+		}
 		matching_user = NULL;
 		runas_match = runaslist_matches(cs->runasuserlist,
 		    cs->runasgrouplist, &matching_user, NULL);
@@ -227,6 +238,7 @@ sudo_file_lookup(struct sudo_nss *nss, int validated, int pwflag)
 		    if (cmnd_match != UNSPEC) {
 			match = cmnd_match;
 			tags = &cs->tags;
+			timeout = cs->timeout;
 #ifdef HAVE_SELINUX
 			/* Set role and type if not specified on command line. */
 			if (user_role == NULL) {
@@ -301,6 +313,8 @@ sudo_file_lookup(struct sudo_nss *nss, int validated, int pwflag)
     if (match == ALLOW) {
 	SET(validated, VALIDATE_SUCCESS);
 	CLR(validated, VALIDATE_FAILURE);
+	if (timeout > 0)
+	    def_command_timeout = timeout;
 	if (tags != NULL) {
 	    if (tags->nopasswd != UNSPEC)
 		def_authenticate = !tags->nopasswd;
@@ -370,6 +384,27 @@ sudo_file_append_cmnd(struct cmndspec *cs, struct cmndtag *tags,
     if (cs->type)
 	sudo_lbuf_append(lbuf, "TYPE=%s ", cs->type);
 #endif /* HAVE_SELINUX */
+    if (cs->timeout > 0) {
+	char numbuf[(((sizeof(int) * 8) + 2) / 3) + 2];
+	snprintf(numbuf, sizeof(numbuf), "%d", cs->timeout);
+	sudo_lbuf_append(lbuf, "TIMEOUT=%s ", numbuf);
+    }
+    if (cs->notbefore != UNSPEC) {
+	char buf[sizeof("CCYYMMDDHHMMSSZ")];
+	struct tm *tm = gmtime(&cs->notbefore);
+	snprintf(buf, sizeof(buf), "%04d%02d%02d%02d%02d%02dZ",
+	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+	    tm->tm_hour, tm->tm_min, tm->tm_sec);
+	sudo_lbuf_append(lbuf, "NOTBEFORE=%s ", buf);
+    }
+    if (cs->notafter != UNSPEC) {
+	char buf[sizeof("CCYYMMDDHHMMSSZ")];
+	struct tm *tm = gmtime(&cs->notafter);
+	snprintf(buf, sizeof(buf), "%04d%02d%02d%02d%02d%02dZ",
+	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+	    tm->tm_hour, tm->tm_min, tm->tm_sec);
+	sudo_lbuf_append(lbuf, "NOTAFTER=%s ", buf);
+    }
     if (TAG_CHANGED(setenv)) {
 	tags->setenv = cs->tags.setenv;
 	sudo_lbuf_append(lbuf, tags->setenv ? "SETENV: " : "NOSETENV: ");
@@ -481,6 +516,8 @@ new_long_entry(struct cmndspec *cs, struct cmndspec *prev_cs)
     if (cs->type && (!prev_cs->type || strcmp(cs->type, prev_cs->type) != 0))
 	return true;
 #endif /* HAVE_SELINUX */
+    if (cs->timeout != prev_cs->timeout)
+	return true;
     return false;
 }
 
@@ -553,6 +590,27 @@ sudo_file_display_priv_long(struct passwd *pw, struct userspec *us,
 		if (cs->type)
 		    sudo_lbuf_append(lbuf, "    Type: %s\n", cs->type);
 #endif /* HAVE_SELINUX */
+		if (cs->timeout > 0) {
+		    char numbuf[(((sizeof(int) * 8) + 2) / 3) + 2];
+		    snprintf(numbuf, sizeof(numbuf), "%d", cs->timeout);
+		    sudo_lbuf_append(lbuf, "    Timeout: %s\n", numbuf);
+		}
+		if (cs->notbefore != UNSPEC) {
+		    char buf[sizeof("CCYYMMDDHHMMSSZ")];
+		    struct tm *tm = gmtime(&cs->notbefore);
+		    snprintf(buf, sizeof(buf), "%04d%02d%02d%02d%02d%02dZ",
+			tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+			tm->tm_hour, tm->tm_min, tm->tm_sec);
+		    sudo_lbuf_append(lbuf, "    NotBefore: %s\n", buf);
+		}
+		if (cs->notafter != UNSPEC) {
+		    char buf[sizeof("CCYYMMDDHHMMSSZ")];
+		    struct tm *tm = gmtime(&cs->notafter);
+		    snprintf(buf, sizeof(buf), "%04d%02d%02d%02d%02d%02dZ",
+			tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+			tm->tm_hour, tm->tm_min, tm->tm_sec);
+		    sudo_lbuf_append(lbuf, "    NotAfter: %s\n", buf);
+		}
 		sudo_lbuf_append(lbuf, _("    Commands:\n"));
 	    }
 	    sudo_lbuf_append(lbuf, "\t");
