@@ -82,8 +82,8 @@ typedef void (*sss_sudo_free_values_t)(char**);
 
 struct sudo_sss_handle {
     char *domainname;
-    char *host;
-    char *shost;
+    char *ipa_host;
+    char *ipa_shost;
     struct passwd *pw;
     void *ssslib;
     sss_sudo_send_recv_t fn_send_recv;
@@ -385,7 +385,7 @@ sudo_sss_open(struct sudo_nss *nss)
     debug_decl(sudo_sss_open, SUDOERS_DEBUG_SSSD);
 
     /* Create a handle container. */
-    handle = malloc(sizeof(struct sudo_sss_handle));
+    handle = calloc(1, sizeof(struct sudo_sss_handle));
     if (handle == NULL) {
 	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
 	debug_return_int(ENOMEM);
@@ -447,9 +447,6 @@ sudo_sss_open(struct sudo_nss *nss)
 	debug_return_int(EFAULT);
     }
 
-    handle->domainname = NULL;
-    handle->host = user_runhost;
-    handle->shost = user_srunhost;
     handle->pw = sudo_user.pw;
     nss->handle = handle;
 
@@ -458,7 +455,7 @@ sudo_sss_open(struct sudo_nss *nss)
      * in sssd.conf and use it in preference to user_runhost.
      */
     if (strcmp(user_runhost, user_host) == 0) {
-	if (get_ipa_hostname(&handle->shost, &handle->host) == -1) {
+	if (get_ipa_hostname(&handle->ipa_shost, &handle->ipa_host) == -1) {
 	    sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
 	    free(handle);
 	    debug_return_int(ENOMEM);
@@ -480,7 +477,10 @@ sudo_sss_close(struct sudo_nss *nss)
     if (nss && nss->handle) {
 	handle = nss->handle;
 	sudo_dso_unload(handle->ssslib);
-	free(nss->handle);
+	free(handle->ipa_host);
+	free(handle->ipa_shost);
+	free(handle);
+	nss->handle = NULL;
     }
     debug_return_int(0);
 }
@@ -583,10 +583,11 @@ sudo_sss_checkpw(struct sudo_nss *nss, struct passwd *pw)
 }
 
 static int
-sudo_sss_check_runas_user(struct sudo_sss_handle *handle, struct sss_sudo_rule *sss_rule, int group_matched)
+sudo_sss_check_runas_user(struct sudo_sss_handle *handle, struct sss_sudo_rule *sss_rule, int *group_matched)
 {
-    char **val_array = NULL;
-    char *val;
+    const char *host = handle->ipa_host ? handle->ipa_host : user_runhost;
+    const char *shost = handle->ipa_shost ? handle->ipa_shost : user_srunhost;
+    char *val, **val_array = NULL;
     int ret = false, i;
     debug_decl(sudo_sss_check_runas_user, SUDOERS_DEBUG_SSSD);
 
@@ -602,9 +603,17 @@ sudo_sss_check_runas_user(struct sudo_sss_handle *handle, struct sss_sudo_rule *
 	break;
     case ENOENT:
 	sudo_debug_printf(SUDO_DEBUG_INFO, "sudoRunAsUser: no result.");
+        if (*group_matched == UNSPEC) {
+            /* We haven't check for sudoRunAsGroup yet, check now. */
+	    i = handle->fn_get_values(sss_rule, "sudoRunAsGroup", &val_array);
+            if (i == 0) {
+                *group_matched = false;
+		handle->fn_free_values(val_array);
+            }
+        }
 	if (!ISSET(sudo_user.flags, RUNAS_USER_SPECIFIED))
 	    debug_return_int(UNSPEC);
-	switch (group_matched) {
+	switch (*group_matched) {
 	case UNSPEC:
 	    /*
 	     * No runas user or group entries.  Match runas_default
@@ -656,8 +665,8 @@ sudo_sss_check_runas_user(struct sudo_sss_handle *handle, struct sss_sudo_rule *
 	switch (val[0]) {
 	case '+':
 	    sudo_debug_printf(SUDO_DEBUG_DEBUG, "netgr_");
-	    if (netgr_matches(val, def_netgroup_tuple ? handle->host : NULL,
-		def_netgroup_tuple ? handle->shost : NULL, runas_pw->pw_name)) {
+	    if (netgr_matches(val, def_netgroup_tuple ? host : NULL,
+		def_netgroup_tuple ? shost : NULL, runas_pw->pw_name)) {
 		sudo_debug_printf(SUDO_DEBUG_DEBUG, "=> match");
 		ret = true;
 	    }
@@ -754,7 +763,7 @@ sudo_sss_check_runas(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 
     if (ISSET(sudo_user.flags, RUNAS_GROUP_SPECIFIED))
 	group_matched = sudo_sss_check_runas_group(handle, rule);
-    user_matched = sudo_sss_check_runas_user(handle, rule, group_matched);
+    user_matched = sudo_sss_check_runas_user(handle, rule, &group_matched);
 
     debug_return_bool(group_matched != false && user_matched != false);
 }
@@ -762,7 +771,9 @@ sudo_sss_check_runas(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 static bool
 sudo_sss_check_host(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 {
-    char **val_array, *val;
+    const char *host = handle->ipa_host ? handle->ipa_host : user_runhost;
+    const char *shost = handle->ipa_shost ? handle->ipa_shost : user_srunhost;
+    char *val, **val_array;
     int matched = UNSPEC;
     bool negated;
     int i;
@@ -792,9 +803,9 @@ sudo_sss_check_host(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 
 	/* match any or address or netgroup or hostname */
 	if (strcmp(val, "ALL") == 0 || addr_matches(val) ||
-	    netgr_matches(val, handle->host, handle->shost,
+	    netgr_matches(val, host, shost,
 	    def_netgroup_tuple ? handle->pw->pw_name : NULL) ||
-	    hostname_matches(handle->shost, handle->host, val)) {
+	    hostname_matches(shost, host, val)) {
 
 	    matched = negated ? false : true;
 	}
@@ -816,12 +827,13 @@ sudo_sss_check_host(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 static bool
 sudo_sss_check_user(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 {
-    int ret = false;
+    const char *host = handle->ipa_host ? handle->ipa_host : user_runhost;
+    const char *shost = handle->ipa_shost ? handle->ipa_shost : user_srunhost;
     char **val_array;
-    int i;
+    int i, ret = false;
     debug_decl(sudo_sss_check_user, SUDOERS_DEBUG_SSSD);
 
-    if (!handle || !rule)
+    if (rule == NULL)
 	debug_return_bool(false);
 
     switch (handle->fn_get_values(rule, "sudoUser", &val_array)) {
@@ -844,8 +856,8 @@ sudo_sss_check_user(struct sudo_sss_handle *handle, struct sss_sudo_rule *rule)
 	switch (*val) {
 	case '+':
 	    /* Netgroup spec found, check membership. */
-	    if (netgr_matches(val, def_netgroup_tuple ? handle->host : NULL,
-		def_netgroup_tuple ? handle->shost : NULL, handle->pw->pw_name)) {
+	    if (netgr_matches(val, def_netgroup_tuple ? host : NULL,
+		def_netgroup_tuple ? shost : NULL, handle->pw->pw_name)) {
 		ret = true;
 	    }
 	    break;
@@ -1059,10 +1071,8 @@ sudo_sss_extract_digest(char **cmnd, struct sudo_digest *digest)
 		    *cmnd = cp;
 		    sudo_debug_printf(SUDO_DEBUG_INFO,
 			"%s digest %s for %s",
-			digest_type == SUDO_DIGEST_SHA224 ? "sha224" :
-			digest_type == SUDO_DIGEST_SHA256 ? "sha256" :
-			digest_type == SUDO_DIGEST_SHA384 ? "sha384" :
-			"sha512", digest->digest_str, cp);
+			digest_type_to_name(digest_type),
+			digest->digest_str, cp);
 		    debug_return_ptr(digest);
 		}
 	    }
@@ -1728,6 +1738,20 @@ sudo_sss_display_entry_short(struct sudo_sss_handle *handle,
 		sudo_lbuf_append(lbuf, negated ? "NOSETENV: " : "SETENV: ");
 	    else if (strcmp(val, "mail_all_cmnds") == 0 || strcmp(val, "mail_always") == 0)
 		sudo_lbuf_append(lbuf, negated ? "NOMAIL: " : "MAIL: ");
+	    else if (!negated && strncmp(val, "command_timeout=", 16) == 0)
+		sudo_lbuf_append(lbuf, "TIMEOUT=%s ", val + 16);
+#ifdef HAVE_SELINUX
+	    else if (!negated && strncmp(val, "role=", 5) == 0)
+		sudo_lbuf_append(lbuf, "ROLE=%s ", val + 5);
+	    else if (!negated && strncmp(val, "type=", 5) == 0)
+		sudo_lbuf_append(lbuf, "TYPE=%s ", val + 5);
+#endif /* HAVE_SELINUX */
+#ifdef HAVE_PRIV_SET
+	    else if (!negated && strncmp(val, "privs=", 6) == 0)
+		sudo_lbuf_append(lbuf, "PRIVS=%s ", val + 6);
+	    else if (!negated && strncmp(val, "limitprivs=", 11) == 0)
+		sudo_lbuf_append(lbuf, "LIMITPRIVS=%s ", val + 11);
+#endif /* HAVE_PRIV_SET */
 	}
 	handle->fn_free_values(val_array);
 	break;
