@@ -36,7 +36,7 @@
 #include "sudo.h"
 #include "sudo_exec.h"
 
-#ifdef _PATH_SUDO_NOEXEC
+#ifdef RTLD_PRELOAD_VAR
 /*
  * Add a DSO file to LD_PRELOAD or the system equivalent.
  */
@@ -144,15 +144,15 @@ preload_dso(char *envp[], const char *dso_file)
 
     debug_return_ptr(envp);
 }
-#endif /* _PATH_SUDO_NOEXEC */
+#endif /* RTLD_PRELOAD_VAR */
 
 /*
  * Disable execution of child processes in the command we are about
  * to run.  On systems with privilege sets, we can remove the exec
  * privilege.  On other systems we use LD_PRELOAD and the like.
  */
-static char **
-disable_execute(char *envp[])
+char **
+disable_execute(char *envp[], const char *dso)
 {
     debug_decl(disable_execute, SUDO_DEBUG_UTIL)
 
@@ -166,10 +166,10 @@ disable_execute(char *envp[])
     sudo_warn(U_("unable to remove PRIV_PROC_EXEC from PRIV_LIMIT"));
 #endif /* HAVE_PRIV_SET */
 
-#ifdef _PATH_SUDO_NOEXEC
-    if (sudo_conf_noexec_path() != NULL)
-	envp = preload_dso(envp, sudo_conf_noexec_path());
-#endif /* _PATH_SUDO_NOEXEC */
+#ifdef RTLD_PRELOAD_VAR
+    if (dso != NULL)
+	envp = preload_dso(envp, dso);
+#endif /* RTLD_PRELOAD_VAR */
 
     debug_return_ptr(envp);
 }
@@ -187,7 +187,7 @@ sudo_execve(int fd, const char *path, char *const argv[], char *envp[], bool noe
 
     /* Modify the environment as needed to disable further execve(). */
     if (noexec)
-	envp = disable_execute(envp);
+	envp = disable_execute(envp, sudo_conf_noexec_path());
 
 #ifdef HAVE_FEXECVE
     if (fd != -1)
