@@ -84,8 +84,8 @@ char *
 tgetpass(const char *prompt, int timeout, int flags,
     struct sudo_conv_callback *callback)
 {
-    sigaction_t sa, savealrm, saveint, savehup, savequit, saveterm;
-    sigaction_t savetstp, savettin, savettou;
+    struct sigaction sa, savealrm, saveint, savehup, savequit, saveterm;
+    struct sigaction savetstp, savettin, savettou;
     char *pass;
     static const char *askpass;
     static char buf[SUDO_CONV_REPL_MAX + 1];
@@ -157,7 +157,7 @@ restart:
      */
     memset(&sa, 0, sizeof(sa));
     sigemptyset(&sa.sa_mask);
-    sa.sa_flags = SA_INTERRUPT;	/* don't restart system calls */
+    sa.sa_flags = 0;	/* don't restart system calls */
     sa.sa_handler = tgetpass_handler;
     (void) sigaction(SIGALRM, &sa, &savealrm);
     (void) sigaction(SIGINT, &sa, &saveint);
@@ -197,17 +197,8 @@ restore:
 
     /* Restore old tty settings. */
     if (!ISSET(flags, TGP_ECHO)) {
-	for (;;) {
-	    /* Restore old tty settings if possible. */
-	    if (sudo_term_restore(input, 1) || errno != EINTR)
-		break;
-	    /* Received SIGTTOU, suspend the process. */
-	    signo[SIGTTOU] = 0;
-	    if (suspend(SIGTTOU, callback) == -1) {
-		pass = NULL;
-		break;
-	    }
-	}
+	/* Restore old tty settings if possible. */
+	(void) sudo_term_restore(input, true);
     }
     if (input != STDIN_FILENO)
 	(void) close(input);
@@ -247,10 +238,18 @@ static char *
 sudo_askpass(const char *askpass, const char *prompt)
 {
     static char buf[SUDO_CONV_REPL_MAX + 1], *pass;
+    struct sigaction sa, savechld;
     int pfd[2], status;
     pid_t child;
     debug_decl(sudo_askpass, SUDO_DEBUG_CONV)
 
+    /* Set SIGCHLD handler to default since we call waitpid() below. */
+    memset(&sa, 0, sizeof(sa));
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = SA_RESTART;
+    sa.sa_handler = SIG_DFL;
+    (void) sigaction(SIGCHLD, &sa, &savechld);
+
     if (pipe(pfd) == -1)
 	sudo_fatal(U_("unable to create pipe"));
 
@@ -297,6 +296,9 @@ sudo_askpass(const char *askpass, const char *prompt)
     if (pass == NULL)
 	errno = EINTR;	/* make cancel button simulate ^C */
 
+    /* Restore saved SIGCHLD handler. */
+    (void) sigaction(SIGCHLD, &savechld, NULL);
+
     debug_return_str_masked(pass);
 }
 
